 /*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code in this file as it was released upon
 * August 13, 2003.
 * 
 * The Initial Developer of the Original Code is Henk-Johan van Rantwijk.
 * all Portions created by Henk-Johan van Rantwijk are Copyright (C) 2003
 * by Henk-Johan van Rantwijk. All Rights Reserved.
 * 
 * Contributor(s):
 *  - none :D
*/

// Our gobal stuff goed here
var tree = null;
var tabbrowser = null;
var stringBundle = null;

////////////////////////////////////////////////////////////////////////////////
function getLastActiveBrowserWindow()
{
  var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService();
  var windowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator);
  var window = windowManagerInterface.getMostRecentWindow("navigator:browser");
  return window;
}
////////////////////////////////////////////////////////////////////////////////
function tabListInit()
{
  var tabList = tabbrowser.mTabList;
  var tabs = tabList.length;
  var URL = null;

  for (var i = 0; i < tabs; i++) {
    var tab = tabList[i];
    var browser = tabbrowser.getBrowserForTab(tab);
    var favicon = tab.getAttribute("image");
    if (tab.hasAttribute("busy"))
      favicon = "chrome://multiviews/skin/icons/color/red.gif";
    else if (!favicon)
      favicon = "chrome://multiviews/skin/icons/color/green.gif";
    var label = tab.getAttribute("label");

    try {
      URL = browser.webNavigation.currentURI.spec;
    } catch(ex) {
      URL = tab.getAttribute("firstpage");
    }
    createCell((i+1), favicon, label, URL, tab.ordinal);
  }
  updateCurrentIndex();
}
////////////////////////////////////////////////////////////////////////////////
function createCell(aNumber, aImage, aLabel, aURL, aOrdinal)
{
  var treeKids = document.getElementById("tabList");
  var row      = document.createElement("treerow");
  var tab      = document.createElement("treecell");
  var label    = document.createElement("treecell");
  var URL      = document.createElement("treecell");
  var ordinal  = document.createElement("treecell");
  var item     = document.createElement("treeitem");

  tab.setAttribute("label", aNumber);
  label.setAttribute("label", aLabel);
  label.setAttribute("src", aImage);
  URL.setAttribute("label", aURL);
  ordinal.setAttribute("label", aOrdinal);
  
  row.appendChild(tab);
  row.appendChild(label);
  row.appendChild(URL);
  row.appendChild(ordinal);

  item.setAttribute("id", "tm-tab-"+aNumber);
  item.appendChild(row);

  treeKids.appendChild(item);
}
////////////////////////////////////////////////////////////////////////////////
function updateCurrentIndex()
{
  tree.currentIndex = tabbrowser.selectedTab.ordinal;
  tree.treeBoxObject.selection.toggleSelect(tree.currentIndex);

  // http://lxr.mozilla.org/seamonkey/source/layout/xul/base/src/tree/public/nsITreeSelection.idl#108
  // if (tree.treeBoxObject.selection.single) alert("single");
  // alert(tree.treeBoxObject.selection.count); // seltype="single"
  // if (tree.treeBoxObject.selection.isSelected(tree.currentIndex)) alert("isSelected");
  // tree.treeBoxObject.selection.select(1); // seltype="single"
  // tree.treeBoxObject.selection.timedSelect(2, 500);
  // tree.treeBoxObject.selection.toggleSelect(tree.currentIndex);
  // tree.treeBoxObject.selection.rangedSelect(0, 4, false); // seltype="multiple"
  // tree.treeBoxObject.selection.clearRange(1, 3); seltype="multiple"
  // tree.treeBoxObject.selection.clearSelection(); // for a single row

  /* tree.treeBoxObject.selection.invertSelection();
  Error: [Exception... "Component returned failure code: 0x80004001 (NS_ERROR_NOT_IMPLEMENTED) [nsITreeSelection.invertSelection]"  nsresult: "0x80004001 (NS_ERROR_NOT_IMPLEMENTED)"  location: "JS frame :: chrome://multiviews/content/multiviewsTabList.js :: updateCurrentIndex :: line 102"  data: no]
  Source File: chrome://multiviews/content/multiviewsTabList.js
  Line: 102 
  See bug: http://bugzilla.mozilla.org/show_bug.cgi?id=216339 */

  // tree.treeBoxObject.selection.selectAll(); // seltype="multiple"
  // alert(tree.treeBoxObject.selection.getRangeCount()); seltype="multiple"
  // alert(tree.treeBoxObject.selection.getRangeAt(0, out long min, out long max));
  // tree.treeBoxObject.selection.invalidateSelection();
  // tree.treeBoxObject.selection.adjustSelection(2, 4); seltype="multiple"
  // alert(tree.treeBoxObject.selection.selectEventsSuppressed);
  // alert(tree.treeBoxObject.selection.currentIndex);
  // alert(tree.treeBoxObject.selection.shiftSelectPivot);

  // http://lxr.mozilla.org/seamonkey/source/layout/xul/base/src/tree/public/nsITreeBoxObject.idl#110
  // http://lxr.mozilla.org/seamonkey/source/layout/xul/base/src/tree/public/nsITreeView.idl#119
  // alert(tree.treeBoxObject.view.canDropOn(0));
  // alert(tree.treeBoxObject.view.canDropBeforeAfter (0, false));
}
////////////////////////////////////////////////////////////////////////////////
function activateSelectedTab()
{
  var rangeCount = tree.treeBoxObject.selection.getRangeCount();
  // do we have an active selection?
  if (rangeCount == 1) // -1 is no selection active
    tabbrowser.selectedTab = tabbrowser.mTabList[rangeCount];
  window.close();
}
////////////////////////////////////////////////////////////////////////////////
function onHelp()
{
  var tabManagerHelpURL = stringBundle.getString("helpPages-TabManager");
  // We could/should sheck the on-line state!
  tabbrowser.loadURI(tabManagerHelpURL);
  window.close();
}
////////////////////////////////////////////////////////////////////////////////
function onCancel()
{
  window.close();
}
////////////////////////////////////////////////////////////////////////////////
function onAccept()
{
  var rangeCount = tree.treeBoxObject.selection.getRangeCount();

  if (rangeCount >= 1) {
    if (tabbrowser.confirm("tabs.confirmDialogTitle", "tabs.confirmDialogText"))
      return doRemove();
  }
  else
    activateSelectedTab();
  window.close();
}
////////////////////////////////////////////////////////////////////////////////
function doSelect(aSelection)
{
  var rows = tree.treeBoxObject.view.rowCount;

  if (aSelection == "all") {
    tree.treeBoxObject.selection.selectAll();
  }
  else if (aSelection == "none") {
    tree.treeBoxObject.selection.clearSelection();
    tree.treeBoxObject.selection.select(tree.currentIndex);
  }
  tree.focus();
}
////////////////////////////////////////////////////////////////////////////////
function doRemove(aSelection)
{
  var t = -1;
  var tabs = new Array();

  if (aSelection)
    doSelect(aSelection);

  var rangeCount = tree.treeBoxObject.selection.getRangeCount();

  for (var i = 0; i < rangeCount; i++) {
    var start = {};
    var end = {};
    tree.treeBoxObject.selection.getRangeAt(i, start, end);

    for (var l = start.value; l <= end.value; l++) {
      var index = tree.view.getCellText(l, "tm-ordinal");
      tabs[++t] = tabbrowser.mTabList[index];
      // dump("range: "+i+"loop: "+l+"tab: "+t"\n");
    }
  }
  if (tabs) {
    for (i = (tabs.length-1); i > -1; i--)
      tabbrowser.removeTab(tabs[i]);
    tabs = null;
  }
}
////////////////////////////////////////////////////////////////////////////////
function closeTab()
{
  if (tabbrowser.confirm("tabs.confirmDialogTitle", "tabs.confirmDialogText"))
    tabbrowser.removeTab(tabbrowser.selectedTab);
  tree.focus();
}
////////////////////////////////////////////////////////////////////////////////
//  Work in progress here, doesn't work yet, planned for a next update
var TabListDNDObserver = 
{
  tree : null,
  
  initTabListDNDObserver: function()
  {
    tree = document.getElementById("tabManagerTree");
    tabbrowser = getLastActiveBrowserWindow().getBrowser();
    stringBundle = document.getElementById("tm-properties");
    tabListInit();
  },

  onDragStart: function(aEvent, aXferData, aDragSession)
  {
    // start the row drag operation only when there are at least 2 visible rows
    if (tree.treeBoxObject.view.rowCount > 1) {
      tree.childNodes[1].setAttribute("dragover", "true");
      var ordinal = tree.view.getCellText(tree.currentIndex, "tm-ordinal");
      // dump("onDragStart ordinal: "+ordinal+"\n");
      aXferData.data = new TransferData();
      aXferData.data.addDataForFlavour("text/unicode", ordinal);
      tree.childNodes[1].removeAttribute("dragover");
      // nsDragAndDrop.mDragSession.sourceNode = null;
    }
  },

  onDragOver: function(aEvent, aXferData, aDragSession)
  {
    var X = aEvent.pageX;
    var Y = aEvent.pageY;
    var row = tree.treeBoxObject.getRowAt(X, Y);
    if (row > -1) {
      dump("onDragOver:\n"+row+"\n");
      var ordinal = tree.view.getCellText(row, "tm-tab");
      dump(ordinal+"\n");
      var element = document.getElementById("tm-tab-"+ordinal);
      dump(element.nodeName+"\n");
      nsDragAndDrop.mDragSession.sourceNode = element;
    }
  },

  onDrop: function(aEvent, aXferData, aDragSession)
  {
    dump("onDrop\n"+aEvent.originalTarget.nodeName);
  },

  onDragExit: function(aEvent, aXferData, aDragSession)
  {
    tree.childNodes[1].removeAttribute("dragover");
  },

  getSupportedFlavours: function ()
  {
    // we might want to expand this
    var flavourSet = new FlavourSet();
    flavourSet.appendFlavour("text/unicode");
    return flavourSet;
  },

  canDrop: function (aEvent, aDragSession)
  {
    // allow drops from the Tab Manager only
    var doc = aDragSession.sourceNode.ownerDocument;
    var elem = doc.getElementById("tabManagerWindow");
    return (elem && (elem.getAttribute("windowtype") == "multizilla:tabmanager"));
  } 
};

addEventListener("load", TabListDNDObserver.initTabListDNDObserver, false);